// Lawson Labs, Inc.
// 15 Phoenixville Pike
// Malvern, PA  19355
// 800 321-5355 or 610 725-8800             October 4, 1996  version 5

#include <windows.h>
#include <alloc.h>
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>
#ifdef __cplusplus
	 #define __CPPARGS ...
#else
	 #define __CPPARGS
#endif

#define ST .0002076
#define ICD .000017
#define ISDM .0000212
#define RSC 0
#define HOLD 1
#define BYTES_PER_LONG 4


//typedef for M250Settings struct
typedef struct{
	short    portAddress;		//Address of the parallel port that the Model
										//250 is connected to
	short 	rwDelay;				//Read/Write settling delay for the parallel port
	double	sIntvl;				//Scan interval
	long 	   stampBuffer;		//Size of the host's circular buffer
	short 	interChanDelay;  	//Delay count between scan channels. Each count
										//equals 17 microseconds
	short		numChannels;		//Number of channels in the scan
	short 	average;				//The averaging factor for the Model 250. Must
										//be powers of 2
	short		tScan;				//type of scan.	0 = time-stamped
										//						1 = time-stamped w/ cal
	short		chan[6];			  	//Array of starting and ending control codes
										//on each channel
	short		triggerType;		//Type of trigger		0 = none
										//							1 = positive edge
										//							1 = negative edge
	double	gainFactor;			//Correction factor for full scale
	double	plusGain;			//Correction factor for positive voltages
										//during scanning. For track/hold only
	double	minusGain;			//Correction factor for negative voltages
										//during scanning. For track/hold only
	short		trackHold;			//Flag for track/hold or DC mode.  0 = DC mode
										//											  1 = track/hold
	short		lastDigInp;			//Last digital input read from the Model 250
	short		version;          //Version of Model 250
	short		channel;			  	//Currently selected channel in polled mode
	short		subChan;				//Currently selected sub channel in polled mode
	double   voltage;				//voltage of last requested conversion in polled mode
	short		digOutWord;			//Digital input word sent to Model 250
	int  		errorType;			//Type of error from Model 250 or DLL
}	T_M250Settings;


//local prototypes

//Make window the same size as the desktop
void    SetWindowToDesktop(void);

//Read configuration file 250DLL.CFG. If tile could not be found,
//set defaults
short   ReadConfig(T_M250Settings far *M250Settings,
						 short *localAvg,
						 char *newCalStr);

//Write configuration file 250DLL.CFG.
short   WriteConfig(T_M250Settings far *M250Settings,
						  short *localAvg,
						  char *newCalStr);

//Interactive setup of a scan. Setup includes scan channels, interchannel
//delay, scan interval, scan buffer size, trigger type, interrupt rate,
//number of scans to get at a time and type of scan
short   SetupStampScans(T_M250Settings far *M250Settings,
								long far *ScanBuffer,
								HANDLE hglbScanBuff);


//Get starting & ending control codes for a scanning channel
short   GetChannel(int value);

//Calculate time to average for scanning
double  GetAvgTime(int average);

//Calculate the minimum interval for time-stamp scanning. Returns the
//minimum scan interval
double  CalcStampMin(int interChan, int average, short numChannels);

//Get and check the time-stamp scan interval using CalcStampMin() and
//CalcStampInterval
void    GetStampInterval(T_M250Settings far *M250Settings);

//Check and calculate the scan interval. Returns the scan interval
double  CalcStampInterval(double sIntvl, double stampMin);

//Set the scan interval. Returns 0 if interval is out of limits
short   SetStampInterval(T_M250Settings far *M250Settings);

//Gets the interchannel delay for scanning
void    GetInterChannelDelay(T_M250Settings far *M250Settings);

//Displays the current calibration file. Load/select new calibration files.
void    LoadCalibrations(T_M250Settings far *M250Settings, char *newCalStr);

//Gets the selected calibration file and returns it in newCalStr
void    ChangeId(char *newCalStr, char *currentCalStr);

//Set the scan buffer size. Returns the scan buffer size
long    SetStampBuffer(long stampBuffer,
							  short numChannels,
							  long far *ScanBuffer,
							  HANDLE hglbScanBuff);

//Procedure that displays the polled mode command menu
void PrintCommandMenu(T_M250Settings far *M250Settings,
							 char Ch,
							 short localAvg);

//Procedure that performs the selected command
void    DoCommand(T_M250Settings far *M250Settings,
						char Ch,
						double *max,
						double *min,
						short *localAvg);

//Standard deviation calculation for the last 40 voltages
void    StandardDev(T_M250Settings far *M250Settings,
						  double *stDv,
						  double *rmsBits,
						  double *sdTot,
						  double *runningAvg,
						  double *sd,
						  int    *sdCount);

//Procedure to log scans to disk.
void LogData(T_M250Settings far *M250Settings,
				 int numberOfScans,
				 int *i,
				 short  *cc,
				 long far *ScanBuffer);

//prototypes for external dll functions

//Get digital input. Digital input is passed back in M250Settings.lastDigInp.
//Returns 0 if an error occurred.
short (FAR PASCAL *GetDigIn)(T_M250Settings far *M250Settings);

//Get Model 250 version. Version is passed back in M250Settings.version.
//Returns 0 if an error occurred.
short (FAR PASCAL *GetVer)(T_M250Settings far *M250Settings);

//Set parallel port. Returns 0 if port could not be found or Model 250 could
//not be found or initialized
short (FAR PASCAL *SetPort)(T_M250Settings far *M250Settings);

//Check parallel port communications. If errors occur, procedure will attempt
//to correct by increasing the read/write delay factor. Returns 0 on failure.
short (FAR PASCAL *PortCheck)(T_M250Settings far *M250Settings);

//Set digital output in digOutWord. Returns 0 if an error occurred.
short (FAR PASCAL *SetDigOut)(short digOutWord);

//Sets track/hold or DC mode. Returns 0 if an error occurred.
short (FAR PASCAL *TrackHold)(short track_hold);

//Sets averaging factor. Factor must be a power of 2. Returns 0 if an error occurred.
short (FAR PASCAL *RemoteAvg)(short average);

//Full scale calibration on the current channel. Returns 0 if an error occurred.
short (FAR PASCAL *FullScaleCal)(T_M250Settings far *M250Settings);

//Offset calibration on the current channel. Returns 0 if an error occurred.
short (FAR PASCAL *OffsetCal)(T_M250Settings far *M250Settings);

//Converts a count to volts. Passes back result in volts.
void  (FAR PASCAL *Counts2Volts)(long counts, double gainFactor, double far *volts);

//Get conversion on current channel. Passes back result in M250Settings.voltage.
//Returns 0 if an error occurred.
short (FAR PASCAL *GetConversion)(T_M250Settings far *M250Settings);

//Gets and compares Model 250 checksum to host checksum. Returns 0 if checksums
//do not agree.
short (FAR PASCAL *GetChecksum)(void);

//Sets the channel and sub channel in M250Settings.channel and
//M250Settings.subChan. Returns 0 if an error occurred.
short (FAR PASCAL *SetChanCode)(T_M250Settings far *M250Settings);

//Get and load calibration files from disk. Uses the name in newCalStr
//Returns 0 if an error occurred. If an error occurred, the error type is
//passed back in M250Settings.errorType. Possible error codes are:
//(M250Settings.errorType && 0x0FF) = 1  	Calibration file not found
//(M250Settings.errorType && 0x0FF) = 2  	Calibration file wrong size, not
//												  		loaded
//(M250Settings.errorType \ 0x100) = 1   	Factory calibration file not found
//(M250Settings.errorType \ 0x100) = 2  	Factory calibration file wrong
//														size, not loaded
short (FAR PASCAL *GetCalibrations)(T_M250Settings far *M250Settings, char *newCalStr);

//Procedure returns the number of scans in the parallel port buffer.
long  (FAR PASCAL *GetNumScans)(T_M250Settings far *M250Settings);

//Procedure will verify and start the scan that was defined in SetupStampScans().
//Returns 0 if an error occurred. If an error occurred, the error type is
//passed back in M250Settings.errorType. Possible error codes are:
//M250Settings.errorType = 1  	No channels selected for scan.
//M250Settings.errorType = 2		Port address incorrect or Model 250 not found
//M250Settings.errorType = 3  	Track/Hold command failed.
//M250Settings.errorType = 4  	Remote average command failed.
//M250Settings.errorType = 5  	Calibration failed.
//M250Settings.errorType = 6  	Gain correction calibration scan failed.
//M250Settings.errorType = 7  	Set time stamp interval failed.
//M250Settings.errorType = 8  	Set scan channels failed.
//M250Settings.errorType = 9  	Send scan setup failed.
//M250Settings.errorType = 10 	Start time-stamped scan command failed.
short (FAR PASCAL *StartScanning)(T_M250Settings far *M250Settings);

//Procedure will get one scan. Returns 0 if an error occurred. If an error
//occurred, the error type is passed back in M250Settings.ErrorType. Possible
//error codes are:
//M250Settings.errorType = 0x1B  Buffer full, recovering
//M250Settings.errorType = 0x1C  No scans available
//M250Settings.errorType = 0x1E	Missed scan(s)
//M250Settings.errorType = 0x20  Buffer overflow, scanning stopped.
short (FAR PASCAL *GetOneScan)(T_M250Settings far *M250Settings, long far *clientDataBuffer);

//Procedure to stop scanning. Procedure will restore the number of timer
//interrupts per second, restore the old ISR, and stop the scan. Returns 0
//if an error occurred.
short (FAR PASCAL *StopScanning)(void);

//Set the interrupt rate in Hz for the ISR.
void (FAR PASCAL *SetInterruptRate)(long intRate);

//Procedure will get the interrupt rate that will be set when the ISR is
//installed.
long (FAR PASCAL *GetInterruptRate)(T_M250Settings far *M250Settings);

//Procedure will get numScan scans. Returns 0 if an error occurred. If an error
//occurred, the error type is passed back in M250Settings.ErrorType. Possible
//error codes are:
//M250Settings.errorType = 0x1B  Buffer full, recovering
//M250Settings.errorType = 0x1C  No scans available
//M250Settings.errorType = 0x1E	Missed acan(s)
//M250Settings.errorType = 0x20  Buffer overflow, scanning stopped.
//If the error type is 0x1B, 0x1C, or 0x1E, the procedure will pass back the
//number of scans it got in completed. Otherwise, if no error occurred, the
//procedure will have stored numScan scans in clientDataBuffer.
short (FAR PASCAL *GetManyScans)(T_M250Settings far *M250Settings,
											long far *clientDataBuffer,
											int numScans,
											long *completed);

//Procedure will calculate the delay time for the ISR and set the delay loop
//counter. CalcDelayLoop() is called by the DLL when it is loaded.
long (FAR PASCAL *CalcDelayLoop)(void);

//Procedure will set the delay loop counter. The procedure SetDelayLoop()
//can be used to set custom delay loop values. Each count is worth 10 clocks
//on the CPU.
long (FAR PASCAL *SetDelayLoop)(unsigned short delay);

//Global Variables
HINSTANCE 	hLibrary;
int numScans=1, getMany = 0;
unsigned short delayLoop = 1;
FILE 	       			*out;

int main(void)
{
long 						timestamp,counts,longBuffSize;
int 						i,n,p_row = 14, scanStart, j, logging;

int                  stream;    	//.temp.

int 						startCC, endCC, ccode;
long far 				*ScanBuffer;
HANDLE 		         hglbScanBuff;
double 					voltage, max=-12000, min=12000;
char 						Ch=0;
char 	 					newCalStr[13]="250A9514.CAL\0";
char 						fileName[12];
T_M250Settings 		M250Settings;
long						lastScans, scanCount = 0, missedScans = 0;
short						receiveError=0, localAvg=1;
char        			tmpbuf[80];
short 					needSignon = 1, tempChan, tempSub;
short						cc[6];
//variables for standard deviation calculation
double               stDv=0, rmsBits=0, sdTot = 0, runningAvg=0, sd[41];;
int    					sdCount=1;
long                 completed = 0;


	 SetWindowToDesktop();

	 //When the DLL is loaded, it will call the procedure CalcDelayLoop() and
	 //establish the CPU's timing
	 clrscr();
	 gotoxy(25,11);
	 printf ("Calculating timing...");

	 if (!ReadConfig(&M250Settings, &localAvg, &newCalStr[0]))
	 {
		gotoxy(24,10);
		printf("unable to read 250DLL.CFG file");
		gotoxy(26,11);
		printf("press any key to continue");
		getch();
	 }


	longBuffSize = (M250Settings.numChannels + 1) * M250Settings.stampBuffer;

	hglbScanBuff = GlobalAlloc(GMEM_FIXED,(longBuffSize * BYTES_PER_LONG));
	ScanBuffer = (long far *)GlobalLock(hglbScanBuff);

	hLibrary = LoadLibrary ("250dLL.dll");
	(FARPROC) GetDigIn =GetProcAddress(hLibrary, "GetDigIn");
	(FARPROC) GetVer =GetProcAddress(hLibrary, "GetVer");
	(FARPROC) SetPort =GetProcAddress(hLibrary, "SetPort");
	(FARPROC) PortCheck =GetProcAddress(hLibrary, "PortCheck");
	(FARPROC) SetDigOut =GetProcAddress(hLibrary, "SetDigOut");
	(FARPROC) TrackHold =GetProcAddress(hLibrary, "TrackHold");
	(FARPROC) RemoteAvg =GetProcAddress(hLibrary, "RemoteAvg");
	(FARPROC) FullScaleCal =GetProcAddress(hLibrary, "FullScaleCal");
	(FARPROC) OffsetCal =GetProcAddress(hLibrary, "OffsetCal");
	(FARPROC) Counts2Volts =GetProcAddress(hLibrary, "Counts2Volts");
	(FARPROC) GetConversion =GetProcAddress(hLibrary, "GetConversion");
	(FARPROC) GetChecksum  =GetProcAddress(hLibrary, "GetChecksum");
	(FARPROC) SetChanCode=GetProcAddress(hLibrary, "SetChanCode");
	(FARPROC) GetNumScans  =GetProcAddress(hLibrary, "GetNumScans");
	(FARPROC) GetCalibrations =GetProcAddress(hLibrary, "GetCalibrations");
	(FARPROC) StartScanning =GetProcAddress(hLibrary, "StartScanning");
	(FARPROC) GetOneScan =GetProcAddress(hLibrary, "GetOneScan");
	(FARPROC) StopScanning =GetProcAddress(hLibrary, "StopScanning");
	(FARPROC) SetInterruptRate =GetProcAddress(hLibrary, "SetInterruptRate");
	(FARPROC) GetInterruptRate =GetProcAddress(hLibrary, "GetInterruptRate");
	(FARPROC) GetManyScans =GetProcAddress(hLibrary, "GetManyScans");
	(FARPROC) SetDelayLoop =GetProcAddress(hLibrary, "SetDelayLoop");
	(FARPROC) CalcDelayLoop =GetProcAddress(hLibrary, "CalcDelayLoop");

  if (!GetCalibrations(&M250Settings, &newCalStr[0]))
  {

	 for(i=3;i<8;i++)
	 {
		newCalStr[i] = ' ';
	 }

  }

TOP_MENU:

  tempChan = M250Settings.channel;
  tempSub = M250Settings.subChan;

  while(Ch != 27)
  {
	 clrscr();
	 gotoxy(18,4);
	 printf ("MODEL 250 23-Bit Data Acquisition System");
	 gotoxy(34,6);
	 printf ("Main Menu");
	 gotoxy(20,8);
	 printf ("1) Parallel Port          %X",M250Settings.portAddress);
	 gotoxy(20,10);
	 printf ("2) Check Port");
	 gotoxy(20,12);
	 printf ("3) Load Calibration Files");
	 gotoxy(20,14);
	 printf ("4) Polled Mode Operation");
	 gotoxy(20,16);
	 printf ("5) Scanning Mode Operation");
	 gotoxy(18,18);
	 printf ("Esc) Exit program");
	 Ch = getch();

	 if (Ch == 27)
	 {
		goto EXIT_POINT;
	 }

	 if (Ch == '1')
	 {

		while(Ch != 'Y' && Ch != 'y')
		{

		  if (M250Settings.portAddress == 0x278)
		  {
			 M250Settings.portAddress = 0x378;
		  }
		  else if (M250Settings.portAddress == 0x378)
		  {
			 M250Settings.portAddress = 0x3BC;
		  }
		  else
		  {
			 M250Settings.portAddress = 0x278;
		  }

		  clrscr();
		  gotoxy(20,5);
		  printf("Is %X the correct port (Y/N)? ",M250Settings.portAddress);
		  Ch = getch();
		  needSignon = 1;
		}  //while(Ch != 'Y' && Ch != 'y')

		gotoxy(20,20);
		printf("                                ");

		if (!SetPort(&M250Settings))
		{
		  MessageBox(NULL,"Recheck port address","Model 250 not found",MB_ICONEXCLAMATION + MB_OK);
		  goto TOP_MENU;
		}
		Ch = 0;
	 }
	 else
	 {

		if (needSignon)
		{

		  if (SetPort(&M250Settings))
		  {

			 if (!GetVer(&M250Settings))
			 {
				MessageBox(NULL,"Couldn't Get Version \n, Check Connections and Power","Port Error",MB_ICONEXCLAMATION + MB_OK);
				goto TOP_MENU;
			 }

			 if (!TrackHold(M250Settings.trackHold))
			 {
				MessageBox(NULL,"Couldn't Set Track or Hold","Port Error",MB_ICONEXCLAMATION + MB_OK);
				goto TOP_MENU;
			 }

			 if (!RemoteAvg(M250Settings.average))
			 {
				MessageBox(NULL,"Couldn't Set Average","Port Error",MB_ICONEXCLAMATION + MB_OK);
				goto TOP_MENU;
			 }

			 M250Settings.channel = 7;
			 M250Settings.subChan= 0;

			 if (!OffsetCal(&M250Settings))
			 {
				MessageBox(NULL,"Couldn't Perform Offset Calibration","Port Error",MB_ICONEXCLAMATION + MB_OK);
				goto TOP_MENU;
			 }

			 M250Settings.channel = 6;

			 if (!FullScaleCal(&M250Settings))
			 {
				MessageBox(NULL,"Couldn't Perform Full Scale Calibration","Port Error",MB_ICONEXCLAMATION + MB_OK);
				goto TOP_MENU;
			 }

			 M250Settings.channel = tempChan;
			 M250Settings.subChan = tempSub;
			 needSignon = 0;
		  }
		  else
		  {
			 MessageBox(NULL,"Couldn't Set Parallel Port \n, Check Connections and Power","Port Error",MB_ICONEXCLAMATION + MB_OK);
			 Ch = 'X';
			 needSignon = 1;
		  }
		}

		if (Ch == '2')
		{
		  clrscr();
		  printf ( "Initial port settling delay = %d\n", M250Settings.rwDelay);
		  gotoxy(1,3);
		  printf ( "This should take under a minute. \n");

		  if (PortCheck(&M250Settings))
		  {
			 gotoxy(1,6); printf ( "             PORT OK\n");
			 printf ( "Current port settling delay = %d\n", M250Settings.rwDelay);
			 printf ( "Type new delay or Enter only to leave unchanged ");
			 gets(tmpbuf);

			 if(strlen(tmpbuf))
			 {
				sscanf(tmpbuf,"%d",&M250Settings.rwDelay);
			 }

		  }
		  else
		  {
			 printf ( "Model 250 not found. Press space to continue");
			 M250Settings.rwDelay = 0;
			 Ch = getch();
			 Ch = 'X';
			 needSignon = 1;
		  }

		}

		if (Ch == '3')
		{
		  LoadCalibrations(&M250Settings, &newCalStr[0]);
		}

		if (Ch == '4')
		{
		  goto POLLED;
		}

		if (Ch == '5')
		{
		  goto GET_SCANS;
		}

	 }

  } //while Ch != 27

goto EXIT_POINT;

POLLED:

  clrscr();
  PrintCommandMenu(&M250Settings, Ch, localAvg);
  while (Ch != 27)		//repeat until esc key pressed or an error
  {

	 while (!kbhit())//loop until keyboard hit
	 {
		gotoxy(1,p_row++);

		voltage = 0;

		for (i=0;i<localAvg;i++)
		{

		  if(!GetConversion(&M250Settings))
		  {
			 MessageBox(NULL,"Get Conversion Failed","Port Error",MB_ICONEXCLAMATION + MB_OK);
			 goto TOP_MENU;
		  }

		  voltage += M250Settings.voltage;
		}
		M250Settings.voltage = voltage/localAvg;

		printf ("            %9.3lf",M250Settings.voltage);   //display voltage

		if (max < M250Settings.voltage)
		{
		  max = M250Settings.voltage;               //update minimum
		}

		if (min > M250Settings.voltage)
		{
		  min = M250Settings.voltage;               //update minimum
		}

		StandardDev(&M250Settings,
						&stDv,
						&rmsBits,
						&sdTot,
						&runningAvg,
						sd,
						&sdCount);

		if (p_row == 23)
		{
		  p_row = 14;
		  gotoxy(40,14);
		  printf ("Maximum     %9.3lf mv       ", max);
		  gotoxy (40,15);
		  printf ("Minimum     %9.3lf mv       ", min);
		  gotoxy (40,16);
		  printf ("Delta       %9.3lf mv       ", fabs(max - min));
		  gotoxy (40,17);
		  printf ("Average     %9.3lf mv       ", runningAvg);
		  gotoxy (40,19);
		  printf ("Standard deviation  %9.3lf mv    ", stDv);
		  gotoxy (40,20);
		  printf ("Effective Resolution  %5.1lf   bits", rmsBits);
		  gotoxy (1,14);
		}

	 }   //while (!kbhit())

	 Ch = getch();

	 if (Ch != 27)
	 {
		DoCommand(&M250Settings, Ch, &max, &min, &localAvg);
	 }

  }	//while (Ch != 27)

  Ch = 'X';
  goto TOP_MENU;  //end of polled mode example


GET_SCANS:

	//start of scanned mode example
	if (!SetupStampScans(&M250Settings, ScanBuffer, hglbScanBuff))
	{
	  Ch = 0;
	  goto TOP_MENU;
	}

	for (n=0;n<6;n++)
	{
		cc[n] = (int)fmod((double)M250Settings.chan[n],16.0) - (M250Settings.chan[n]>>4) + 1;

		if ( cc[n] == -7)
		{
			cc[n] = 0;
		}
	}

	logging = 0;

	clrscr();
	gotoxy(20,13);
	printf("Enter name for disk file,");
	gotoxy(20,15);
	printf("or just enter for no file ");
	gets(fileName);

	if (strlen(fileName))
	{
	  logging = 1;
	}

	if(logging)
	{
	  if ((out = fopen(fileName, "at"))== NULL)
	  {
		 gotoxy(20,19);
		 printf("Could not open/create file\n");
		 goto SCAN_STOPPED;
	  }
	}

	clrscr();

	if (M250Settings.triggerType)
	{
	  gotoxy(30,10);
	  printf("Waiting for trigger");
	}

	M250Settings.errorType = 0;

	for (n = 0;n < (M250Settings.stampBuffer*(M250Settings.numChannels + 1));n++)			//.temp.
	{
	  ScanBuffer[n] = 0;
	}

	if (!StartScanning(&M250Settings))
	{
	  printf("%d",M250Settings.errorType);
	  getch();
	  switch (M250Settings.errorType)
	  {
	  case 1:
		 MessageBox(NULL,"Channels must be selected first","ERROR",MB_ICONEXCLAMATION);
		 break;
	  case 2:
		  MessageBox(NULL,"Recheck port address","Model 250 not found",MB_ICONEXCLAMATION + MB_OK);
		 break;
	  case 3:
		 MessageBox(NULL,"Track/Hold command failed","ERROR",MB_ICONEXCLAMATION);
		 break;
	  case 4:
		 MessageBox(NULL,"Remote average command failed","ERROR",MB_ICONEXCLAMATION);
		 break;
	  case 5:
		 MessageBox(NULL,"Calibration failed","ERROR",MB_ICONEXCLAMATION);
		 break;
	  case 6:
		 MessageBox(NULL,"Gain correction calibration scan failed","ERROR",MB_ICONEXCLAMATION);
		 break;
	  case 7:
		 MessageBox(NULL,"Set time stamp interval failed","ERROR",MB_ICONEXCLAMATION);
		 break;
	  case 8:
		 MessageBox(NULL,"Set scan channels failed","ERROR",MB_ICONEXCLAMATION);
		 break;
	  case 9:
		 MessageBox(NULL,"Send scan setup failed","ERROR",MB_ICONEXCLAMATION);
		 break;
	  case 10:
		 MessageBox(NULL,"Start time-stamped scan command failed","ERROR",MB_ICONEXCLAMATION);
		 break;
	  }
	}
	scanCount = 0;
	missedScans = 0;
	receiveError = 0;
	i = 0;
	clrscr();
	gotoxy(20,23);
	printf ("Press any key to stop scanning");

	while(!kbhit())
	{

		if (getMany == 1 && (GetNumScans(&M250Settings) > numScans + 10))
		{

		  completed = 0;
		  receiveError = GetManyScans(&M250Settings,
												ScanBuffer,
												numScans,
												&completed);

		  if(!receiveError)
		  {

			 if(M250Settings.errorType == 0x1C)
			 {

				 if (completed > numScans)
				 {
					gotoxy(20,1);
					printf ("completed error %lu",completed);
				 }

				 gotoxy(20,12);
				 printf ("waiting for next scan");
				 scanCount += completed;
				 gotoxy(20,10);
				 printf( "Number of Scans Received  %lu", scanCount);

				 if (logging && (completed > 0))
				 {
					LogData(&M250Settings, (int)completed, &i, cc, ScanBuffer);
				 }

			 }
			 else if (M250Settings.errorType == 0x1E)
			 {
				 if (completed > numScans)
				 {
					gotoxy(20,2);
					printf ("completed error %lu",completed);
				 }

				 missedScans++;
				 gotoxy(20,13);
				 printf ("missed scans      %lu",missedScans);
				 scanCount += completed;
				 gotoxy(20,10);
				 printf( "Number of Scans Received  %lu", scanCount);
				 if (logging && (completed > 0))
				 {
					LogData(&M250Settings, (int)completed, &i, cc, ScanBuffer);
				 }
			 }
			 else if (M250Settings.errorType == 0x1B)
			 {
				 gotoxy(20,14);
				 printf ("buffer full, recovering");
				 gotoxy(20,8);
				 printf("scans in buffer = %9lu",GetNumScans(&M250Settings));
			 }
			 else if (M250Settings.errorType == 0x20)
			 {
				 MessageBox(NULL,"Scan Buffer Overflow. Scanning Terminated!","Buffer Overflow",MB_ICONEXCLAMATION);
				 gotoxy(20,15);
				 printf ("fatal buffer full, scan stopped");
				 goto SCAN_STOPPED;
			 }

		  }
		  else
		  {
			 scanCount += numScans;
			 gotoxy(20,10);
			 printf( "Number of Scans Received  %lu", scanCount);
			 gotoxy(20,12);
			 printf ("                        ");
			 if (logging)
			 {
				LogData(&M250Settings, numScans, &i, cc, ScanBuffer);
			 }
		  }

		  gotoxy(20,8);
		  printf("scans in buffer   %9lu",GetNumScans(&M250Settings));
		  gotoxy(20,10);
		}
		else if (getMany == 0)
		{
		  receiveError = GetOneScan(&M250Settings, ScanBuffer);

		  if(!receiveError)
		  {

			 if(M250Settings.errorType == 0x1C)
			 {
				 gotoxy(20,12);
				 printf ("waiting for next scan");
			 }
			 else if (M250Settings.errorType == 0x1E)
			 {
				 gotoxy(20,13);
				 printf ("missed scans      %lu",++missedScans);
			 }
			 else if (M250Settings.errorType == 0x1B)
			 {
				 gotoxy(20,14);
				 printf ("buffer full, recovering");
				 gotoxy(20,8);
				 printf("scans in buffer = %9lu",GetNumScans(&M250Settings));
			 }
			 else if (M250Settings.errorType == 0x20)
			 {
				 MessageBox(NULL,"Scan Buffer Overflow. Scanning Terminated!","Buffer Overflow",MB_ICONEXCLAMATION);
				 gotoxy(20,15);
				 printf ("fatal buffer full, scan stopped");
				 goto SCAN_STOPPED;
			 }

		  }
		  else
		  {
			 gotoxy(20,10);
			 printf( "Number of Scans Received  %lu", ++scanCount);
			 gotoxy(20,12);
			 printf ("                        ");
			 if (logging)
			 {
				LogData(&M250Settings, 1, &i, cc, ScanBuffer);
			 }

		  }

		  gotoxy(20,8);
		  printf("scans in buffer   %9lu",GetNumScans(&M250Settings));
		  gotoxy(20,10);
		}

	}  //end while

	if(!StopScanning())
	{
	  MessageBox(NULL,"Couldn't Perform Stop Scanning","Port Error",MB_ICONEXCLAMATION + MB_OK);
	}


SCAN_STOPPED:

	if (logging)
	{
	  fclose(out);
	}

	Ch = getch();
	clrscr();

	i=0;
	lastScans = scanCount;
	scanStart = 0;

	if (scanCount> M250Settings.stampBuffer)
	{
	  scanStart = fmod((scanCount * (M250Settings.numChannels + 1)), (M250Settings.stampBuffer*(M250Settings.numChannels + 1)));
	  lastScans = M250Settings.stampBuffer;
	}

RESTART_SCANS:
  n = 0;
  Ch = 0;
  i = scanStart;
  stream = 0;                     //.temp.

  clrscr();
  printf ("CC    chan 0      chan 1      chan 2     ");
  printf (" chan 3      chan 4      chan 5\n\n");
  for (j = 0;j<=15; j++)
  {
	  printf("%d\n",j);
  }

	while (n<lastScans && Ch != 27)
	{

	  if (kbhit())
	  {
		 Ch = getch();
	  }
	  gotoxy(1,24);

	  if (i == (M250Settings.stampBuffer*(M250Settings.numChannels + 1)))
	  {
		 i = 0;
	  }
	  timestamp = ScanBuffer[i];

	  printf("elapsed time: %11.4lf", (timestamp-1)*M250Settings.sIntvl);

	  if (M250Settings.tScan == 1)
	  {
		 i++;

		 if (i == (M250Settings.stampBuffer*(M250Settings.numChannels + 1)))
		 {
			i = 0;
		 }
		 counts = ScanBuffer[i];
		 voltage = counts * M250Settings.gainFactor *.001192093;

		 if (M250Settings.trackHold)
		 {

			if (voltage < 0)
			{
			  voltage *= M250Settings.minusGain;
			}
			else
			{
			  voltage *= M250Settings.plusGain;
			}

		 }

		 gotoxy(1,22);
		 printf("Channel 6 %9.3lf ",voltage);
	  }

	  for (j = 0;j<= 5;j++)           //get # of datapoints for each channel
	  {
		 startCC = M250Settings.chan[j]>>4;
		 endCC = cc[j] + startCC - 1;
		 for (ccode = startCC;ccode<= endCC; ccode++)
		 {
			 i++;

			 if (i == (M250Settings.stampBuffer*(M250Settings.numChannels + 1)))
			 {
				i = 0;
			 }
			 counts = ScanBuffer[i];
			 voltage = counts * M250Settings.gainFactor *.001192093;

			 if (M250Settings.trackHold)
			 {

				if (voltage < 0)
				{
				  voltage *= M250Settings.minusGain;
				}
				else
				{
					voltage *= M250Settings.plusGain;
				}

			 }

			 gotoxy ((j * 12) + 3,(ccode + 3));
			 printf(" %9.3lf ",voltage);
		 }

	  }

	  gotoxy (1,25);
	  n++;
	  printf ("scan %6d  of %6ld", n, lastScans);
	  i++;
	  printf ("  Press space to continue, R to re-run, ESC to quit");

	  if (stream == 0)                //.temp. (if then around getch)
	  {
		 Ch = getch();
	  }

	  if (Ch == 'N' || Ch == 'n')		//.temp.
	  {
		 Ch = 0;
		 if (stream == 1)
		 {
			stream = 0;
		 }
		 else
		 {
			stream = 1;
		 }
	  }

	  if (Ch == 'R' || Ch == 'r')
	  {
		 goto RESTART_SCANS;
	  }

	}

	while (Ch != 27)
	{
	  gotoxy (23,25);
	  printf ("  Press R to re-run, ESC to quit                     ");
	  Ch = getch();

	  if (Ch == 'R' || Ch == 'r')
	  {
		 goto RESTART_SCANS;
	  }

	}
	goto GET_SCANS;

EXIT_POINT:

	GlobalFree(hglbScanBuff);
	GlobalUnlock(hglbScanBuff);
	FreeLibrary(hLibrary);

	clrscr();
	printf("\nDo you wish to save configuration (y/n)?  ");
	Ch = getch();

	if (Ch == 'Y' || Ch == 'y')
	{
	  WriteConfig(&M250Settings,&localAvg,&newCalStr[0]);
	}

	DestroyWindow(GetActiveWindow());
	exit(0);
	return 0;

}


//   calculate the minimum interval for time-stamp scanning
double CalcStampMin(int interChan, int average, short numChannels)
{
double tempstore, averageTime, scanTime;

	averageTime = GetAvgTime(average);
	scanTime = (ST + .000008 + averageTime + interChan * ICD) * 2 * numChannels;

	tempstore = scanTime + .0001052 + .000062;
	return tempstore;
}


//   get time-stamp scan interval
void GetStampInterval(T_M250Settings far *M250Settings)
{
double stampMax = .000062 * 16777215l, stampMin;
char tmpbuf[80];

	clrscr();
	stampMin = CalcStampMin(M250Settings->interChanDelay,
											M250Settings->average,
											M250Settings->numChannels);

	if (floor(M250Settings->sIntvl) >= 1)
	{
		printf("   MIN %11.6lf sec    MAX %11.6lf sec\n", stampMin, stampMax);
		printf ("Enter time-stamp scan interval in seconds ");
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%lf",&M250Settings->sIntvl);
	}
	else
	{
		printf("   MIN %11.6lf hz    MAX %11.6lf Hz\n", 1/stampMax, 1/stampMin);
		printf ("Enter time-stamp scan rete in hertz ");
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%lf",&M250Settings->sIntvl);

		if (M250Settings->sIntvl < stampMin)
		{
			M250Settings->sIntvl = stampMin;
		}
		else
		{
			M250Settings->sIntvl = 1 / M250Settings->sIntvl;
		}

	}

	if (M250Settings->sIntvl < stampMin)
	{
		M250Settings->sIntvl = stampMin;
	}
	else if (M250Settings->sIntvl > stampMax )
	{
		M250Settings->sIntvl = stampMax;
	}

	M250Settings->sIntvl = CalcStampInterval(M250Settings->sIntvl, stampMin);

}

//   set time-stamp scan interval
short SetStampInterval(T_M250Settings far *M250Settings)
{
double stampMax = .000062 * 16777215l, stampMin;

	stampMin = CalcStampMin(M250Settings->interChanDelay,
									M250Settings->average,
									M250Settings->numChannels);

	if ((M250Settings->sIntvl < stampMin) ||
		 (M250Settings->sIntvl > stampMax))
	{
		clrscr();
		printf( "  %11.6lf       ", M250Settings->sIntvl);

		if (floor(M250Settings->sIntvl) >= 1)
		{
			printf("   MIN %11.6lf sec   MAX %11.6lf sec\n", stampMin, stampMax);
		}
		else
		{
			printf("   MIN %11.6lf hz    MAX %11.6lf Hz\n", 1/stampMax, 1/stampMin);
		}

		gotoxy(1,4);
		printf( "Scan interval out of limits.  Press any key to continue\n");

		getch();

		clrscr();
		GetStampInterval(M250Settings);
		return 0;
	}

	return 1;
}


//   calculate time-stamp scan interval
double CalcStampInterval(double sIntvl, double stampMin)
{
long   stamps;
double interval;
int 	 delayCountLo=20, delayCountMid=0, delayCountHi=0;

	interval = floor(((sIntvl - (stampMin - .000062)) / .000062) + .0000005);

	delayCountHi = floor(interval / 65536l);
	delayCountMid = floor((interval - delayCountHi * 65536l) / 256);
	delayCountLo = floor(interval - (delayCountHi * 65536l + delayCountMid * 256));

	//calc stamp scan interval based on # of whole rollovers
	stamps = ((delayCountHi * 65536l + delayCountMid * 256 + delayCountLo));

	interval = stampMin + (stamps - 1) * .000062;
	return interval;
}

//   get size of circular buffer for time-stamp scanning
long SetStampBuffer(long stampBuffer,
						  short numChannels,
						  long far *ScanBuffer,
						  HANDLE hglbScanBuff)
{
long tempVar= 0, maxBuffer = 65535l, longBuffSize;
char tmpbuf[80];

	clrscr();

	if (stampBuffer > maxBuffer )
	{
		printf( "BUFFER TOO LARGE\n");
	}

	gotoxy(1, 3);
	printf( "Current buffer size: %ld\n", stampBuffer);

	gotoxy(1, 5);
	printf("Enter size of buffer (1 to %ld)  ",maxBuffer);

	fgets(tmpbuf,80,stdin);
	sscanf(tmpbuf,"%ld",&tempVar);

	if (tempVar > maxBuffer)
	{
		tempVar = maxBuffer;
	}
	else if (tempVar < 1 )
	{
		tempVar = 1;
	}

	if (stampBuffer > maxBuffer)
	{
		stampBuffer = maxBuffer;
	}
	stampBuffer = tempVar;

	longBuffSize = (numChannels + 1) * stampBuffer;

	GlobalFree(hglbScanBuff);
	GlobalUnlock(hglbScanBuff);

	hglbScanBuff = GlobalAlloc(GMEM_FIXED,(longBuffSize * BYTES_PER_LONG));
	ScanBuffer = (long far *)GlobalLock(hglbScanBuff);

	if (ScanBuffer == NULL)
	{
	  printf("\nBuffer allocation error\n");
	  getch();
	}

	return stampBuffer;
}


//   if averaging, calculate time to average
double GetAvgTime(int average)
{
double averageTime = 0;


	if (average > 1)
	{
		averageTime = 176.6 * average - 164 + 2.2 * log((double)average) / log(2.0);
		averageTime = averageTime / 1000000l;
	}

	return averageTime;
}


//   get interchannel delay
void GetInterChannelDelay(T_M250Settings far *M250Settings)
{
int interChan=0;
char tmpbuf[80];

	clrscr();

	gotoxy( 1, 9);
	printf( "Delay will be rounded to a multiple of 17 microseconds.\n");

	gotoxy( 1, 3);
	printf( "Current interchannel delay = %d\n",M250Settings->interChanDelay*17);

	gotoxy( 1, 5);
	printf( "Enter interchannel delay 0-4335 ");

	fgets(tmpbuf,80,stdin);
	sscanf(tmpbuf,"%d",&interChan);

	interChan = floor((interChan + 8) / 17);

	if (interChan > 255)
	{
		interChan = 255;
	}

	M250Settings->interChanDelay = interChan;
}


short SetupStampScans(T_M250Settings far *M250Settings,
							 long far *ScanBuffer,
							 HANDLE hglbScanBuff)
{
int n;
short cc[6];
char Ch, tmpbuf[80];
long intRate;

STAMP_SCAN:

	clrscr();

	gotoxy(29,1);
	printf( "SCANNING CONFIGURATION");

	gotoxy(6,3);
	printf( "Channel   External control code");

	gotoxy(40,3);
	printf( "Number of data");

	gotoxy(18,4);
	printf("from        to        points this channel");

	M250Settings->numChannels = 0;       //calculate total number of active channels

	for (n=0;n<6;n++)
	{
		cc[n] = (int)fmod((double)M250Settings->chan[n],16.0) - (M250Settings->chan[n]>>4) + 1;

		if ( cc[n] == -7)
		{
			cc[n] = 0;
		}

		M250Settings->numChannels = M250Settings->numChannels + cc[n];           //# of datapoints

		gotoxy(10,6 + n * 2);
		printf( "%d)", n);
		printf("        %d          ",M250Settings->chan[n] / 16);
		printf("%d              %d",(int)fmod((double)M250Settings->chan[n],16), cc[n]);
	}

	if (M250Settings->tScan == 1)
	{
		M250Settings->numChannels++;
	}

	if (!SetStampInterval(M250Settings))
	{
	  goto STAMP_SCAN;
	}

	gotoxy(10,18);
	printf("6) Interchannel delay               ");
	printf(" %4d   microseconds",M250Settings->interChanDelay*17);

	gotoxy(10,19);

	if (M250Settings->sIntvl > 1)
	{
		printf("7) Scanning interval, seconds      ");
		printf("%13.6lf",M250Settings->sIntvl);
	}
	else
	{
		printf( "7) Scanning rate                   ");

		if (M250Settings->sIntvl != 0)
		{
			printf( "%8.1lf Hz", 1 / M250Settings->sIntvl);
		}

	}

	gotoxy(10,20);
	printf("8) Scan Buffer Size                 %5ld",M250Settings->stampBuffer);

	gotoxy(10,21);
	printf("9) Trigger Type             ");
	if (M250Settings->triggerType == 0)
	{
	  printf ("         none");
	}
	else if (M250Settings->triggerType == 1)
	{
	  printf ("positive edge");
	}
	else
	{
	  printf ("negative edge");
	}

	intRate = GetInterruptRate(M250Settings);
	gotoxy(10,22);
	printf("I) Interrupt Rate                  %6ld   Hz",intRate);

	gotoxy(10,24);
	printf( "Enter) begin scanning");

	gotoxy(35,24);
	printf( "Total points per scan %d",M250Settings->numChannels);

	gotoxy(6,25);
	printf( "T) Toggle scan type: ");

	if (M250Settings->tScan == 0)
	{
		printf ("time-stamp scan");
	}
	else
	{
		printf ("time-stamp scan w/cal");
	}

	gotoxy(52,25);
	printf( "N) Get N Scans: %3d",numScans);

	Ch = getch();								//get user input

	if ( (Ch >= '0') && (Ch <= '5'))
	{
		M250Settings->chan[Ch-'0'] = GetChannel(Ch-'0');
	}
	else if ( Ch == 'T' || Ch == 't')
	{

		if (M250Settings->tScan == 1)
		{
			M250Settings->tScan = 0;
		}
		else
		{
			M250Settings->tScan = 1;
		}

	}
	else if ( Ch == 'I' || Ch == 'i')
	{
	  clrscr();
	  intRate = GetInterruptRate(M250Settings);
	  printf ("Current interrupt rate  = %ld Hz\n",intRate);
	  printf ("Enter desired interrupt rate (0 to leave unchanged) ");
	  fgets(tmpbuf,80,stdin);
	  sscanf(tmpbuf,"%ld",&intRate);

	  if (intRate > 0 )
	  {
		 SetInterruptRate(intRate);
	  }

//Use the following code if you which to enter your own delay values.
//Each count of delayLoop = 10 clocks
//
//   if (delayLoop == 0)
//   {
//	    delayLoop = (unsigned short) CalcDelayLoop();
//	  }
//   printf ("\n\nCurrent delay loop count  = %d\n",delayLoop);
//	  printf ("Enter desired delay loop count (0 to leave unchanged) ");
//	  fgets(tmpbuf,80,stdin);
//	  sscanf(tmpbuf,"%d",&delayLoop);

//	  if (delayLoop > 0 )
//	  {
//		 SetDelayLoop(delayLoop);
//		 printf ("delay time is %d",delayLoop);
//		 getch();
//	  }

	}

	else if ( Ch == 'N' || Ch == 'n')
	{
	  clrscr();
	  printf ("Enter desired number of scans (1 to select one scan) ");
	  fgets(tmpbuf,80,stdin);
	  sscanf(tmpbuf,"%d",&numScans);

	  if (numScans > 1)
	  {
		 getMany = 1;
	  }
	  else
	  {
		 numScans = 1;
		 getMany = 0;
	  }
	}
	else if ( Ch == '6')
	{
		GetInterChannelDelay(M250Settings);
	}
	else if ( Ch == '7')
	{
		GetStampInterval(M250Settings);
	}
	else if ( Ch == '8')
	{
	  M250Settings->stampBuffer =  SetStampBuffer(M250Settings->stampBuffer,
											 M250Settings->numChannels,
											 ScanBuffer,
											 hglbScanBuff);
	}
	else if ( Ch == '9')
	{

		if (M250Settings->triggerType == 0)
		{
			M250Settings->triggerType = 1;
		}
		else if (M250Settings->triggerType == 1)
		{
			M250Settings->triggerType = 2;
		}
		else if (M250Settings->triggerType == 2)
		{
			M250Settings->triggerType = 0;
		}

	}
	else if ( Ch == 13 && M250Settings->numChannels > 0)
	{
	  //make sure that the buffer is the correct size
	  GlobalFree(hglbScanBuff);
	  GlobalUnlock(hglbScanBuff);

	  hglbScanBuff = GlobalAlloc(GMEM_FIXED,
	  (((M250Settings->numChannels + 1) * M250Settings->stampBuffer) * BYTES_PER_LONG));
	  ScanBuffer = (long far *)GlobalLock(hglbScanBuff);

	  return 1;
	}
	else if ( Ch == 27)
	{
		return 0;
	}

	goto STAMP_SCAN;				//added (go back to start of menu)

}

//   Get starting & ending control codes for a scanning channel
short GetChannel(int value)
{
short startCC=0, endCC=0, channel;
char  tmpbuf[80];


GET_CHANNEL:

	clrscr();

	gotoxy(4,23);
	printf( "Enter an end code smaller than the start code to skip this channel");

	gotoxy(1,12);
	printf( "Channel number:    %d\n",value);
	printf("Enter starting control code  ");
	gets(tmpbuf);
	startCC = -1;

	if(strlen(tmpbuf))
	{
	  sscanf(tmpbuf, "%d*c",&startCC);
	}

	if ((startCC < 0) || (startCC > 15))
	{
		goto GET_CHANNEL;
	}


GET_END:

	printf("Enter ending control code  ");
	gets(tmpbuf);
	endCC = -1;

	if(strlen(tmpbuf))
	{
	  sscanf(tmpbuf, "%d*c",&endCC);
	}

	if ((endCC < 0) || (endCC > 15))
	{
		goto GET_END;
	}

	if ( startCC > endCC)
	{
		startCC = 8;
		endCC = 0;
	}

	channel = startCC * 16 + endCC;
	return channel;
}

void PrintCommandMenu(T_M250Settings far *M250Settings,
							 char Ch,
							 short localAvg)
{
int nn;

  gotoxy (10,1);
  printf ("POLLED  MENU   Lawson Labs Model 250   Version=%d\n", M250Settings->version);
  gotoxy (5,4);
  printf ("C) A/D input Channel    %d", M250Settings->channel);
  gotoxy (5, 5);
  printf ("E) External sub-channel %d", M250Settings->subChan);
  gotoxy (5,6);
  printf ("A) remote Averaging     %d", M250Settings->average);
  gotoxy (5,7);
  printf ("L) Local averaging      %d        ", localAvg);
  gotoxy (5,8);
  printf ("T) Track/Hold mode      ");

  if (M250Settings->trackHold == 1)
  {
	 printf ("Track/Hold");
  }
  else
  {
	 printf ("DC Mode   ");
  }

  gotoxy (43,4); printf ("D) Digital inputs ");

  if (Ch == 'D')
  {
	 printf ("%d",M250Settings->lastDigInp);
	 gotoxy (61,3); printf ("Decimal 76543210\n");
	 gotoxy (69,4);

	 for (nn = 7;nn != -1; nn--)
	 {

		if (M250Settings->lastDigInp&(int)(pow(2, nn)))
		{
		  printf ("1");
		}
		else
		{
		  printf ("0");
		}

	 }

  }

  gotoxy (43,5); printf ("U) digital oUtput   ");

  if (Ch == 'U')
  {
	 printf ("%d  ",M250Settings->digOutWord);
  }

  gotoxy (43,6); printf ("K) checKsum         ");

  if (Ch == 'K')
  {
	 printf (" OK");
  }
  else
  {
	 printf ("   ");
  }

  gotoxy (43,7); printf ("S) System calibration");
  gotoxy (43,8); printf ("O) Offset calibration");
  gotoxy (13,12); printf ("Millivolts\n");
  gotoxy (1,24);
  printf (" Press ESC for menu or space to reset max/min");
}


void    StandardDev(T_M250Settings far *M250Settings,
						  double *stDv,
						  double *rmsBits,
						  double *sdTot,
						  double *runningAvg,
						  double *sd,
						  int    *sdCount)
{//standard deviation and RMS resolution calculation
int i = 0, nnn,sdGrp=40;;
double tempStDv, StDvRms;

  sd[*sdCount] = M250Settings->voltage;
  (*sdTot) += M250Settings->voltage;

  if ((*sdCount) == sdGrp)
  {
	 (*stDv) = 0; (*sdTot) = (*sdTot) / sdGrp;

	 for (nnn = 1; nnn != sdGrp; nnn++)
	 {
		(*stDv) = (*stDv) + pow(((*sdTot) - sd[nnn]), 2);
	 }

	 (*stDv) = sqrt((*stDv) / sdGrp);
	 StDvRms = (*stDv) / .001192093;
	 tempStDv = StDvRms;

CALCNEXT:

	 if (tempStDv >= 1)
	 {
		i = i + 1;
		tempStDv = tempStDv / 2;
		goto CALCNEXT;
	 }

	 (*rmsBits) = 23 - i;

	 if ((tempStDv < .75) && (i != 0))
	 {
		(*rmsBits) = (*rmsBits) + .5;
	 }

	 (*runningAvg) = (*sdTot);
	 (*sdTot) = 0; (*sdCount) = 1;
  }
  else
  {
	 (*sdCount)++;
  }

}


void DoCommand(T_M250Settings far *M250Settings,
					char Ch,
					double *max,
					double *min,
					short *localAvg)
{
char tempCh=0;
int tempSubChan=0;
char tmpbuf[80];

  if (islower(Ch))
  {
	 Ch = toupper(Ch);
  }

  gotoxy (1,24);
  printf ("                                                   ");

  if (Ch == 'T')
  {

	 if (M250Settings->trackHold == HOLD)
	 {
		M250Settings->trackHold = 0;
	 }
	 else
	 {
		M250Settings->trackHold = HOLD;
	 }

	 if (!TrackHold(M250Settings->trackHold))
	 {
		MessageBox(NULL,"Couldn't Set Track or Hold","Port Error",MB_ICONEXCLAMATION + MB_OK);
	 }

  }

  if (Ch == 'A')
  {

	 while((tempCh != 27) && (tempCh != 13))
	 {

		while (!kbhit())
		{
		  gotoxy (1,24);
		  printf ( "Press + increase, - decrease averaging, Enter to continue ");
		  printf ( "%d              ", M250Settings->average);
		}

		tempCh = getch();
		if (tempCh == '+')
		{
		  M250Settings->average *= 2;
		}

		if (tempCh == '-')
		{
		  M250Settings->average /= 2;
		}

		if (M250Settings->average  < 1)
		{
		  M250Settings->average = 128;
		}

		if (M250Settings->average > 128)
		{
		  M250Settings->average = 1;
		}

	 }

	 if (!RemoteAvg(M250Settings->average))
	 {
		MessageBox(NULL,"Couldn't Set Average","Port Error",MB_ICONEXCLAMATION + MB_OK);
    }
  }

  if (Ch == 'C')
  {
	 tempCh = -1;

	 while ((tempCh < '0') || (tempCh > '7'))
	 {
		gotoxy(1,24);
		printf("Enter channel ");
		tempCh = getch();
	 }

	 M250Settings->channel = tempCh - '0';
  }

  if (Ch == 'D')
  {

	 if (!GetDigIn(M250Settings))
	 {
		MessageBox(NULL,"Couldn't Get Digital Input","Port Error",MB_ICONEXCLAMATION + MB_OK);
	 }

  }

  if (Ch == 'E')
  {
	 tempSubChan = -1;

	 while ((tempSubChan < 0) || (tempSubChan > 15))
	 {
		gotoxy(1,24);
		printf("Enter sub channel ");
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%d",&tempSubChan);
	 }

	 M250Settings->subChan = tempSubChan;
  }

  if (Ch == 'L')
  {
	 (*localAvg) = 0;
	 while ((*localAvg) < 1)
	 {
		gotoxy(1,24);
		printf("Enter local averaging ");
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%d",localAvg);
	 }

  }

  if (Ch == 'K')
  {

	 if (!GetChecksum())
	 {
		MessageBox(NULL,"Checksum Incorrect","Port Error",MB_ICONEXCLAMATION + MB_OK);
	 }

  }

  if (Ch == 'S')
  {
	 tempCh =M250Settings->channel;

	 M250Settings->channel = 7;

	 if (!OffsetCal(M250Settings))
	 {
		MessageBox(NULL,"Couldn't Perform Offset Calibration","Port Error",MB_ICONEXCLAMATION + MB_OK);
	 }

	 M250Settings->channel = 6;

	 if (!FullScaleCal(M250Settings))
	 {
		MessageBox(NULL,"Couldn't Perform Full Scale Calibration","Port Error",MB_ICONEXCLAMATION + MB_OK);
	 }

	 M250Settings->channel=tempCh;
  }

  if (Ch == 'O')
  {

	 if (!OffsetCal(M250Settings))
	 {
		MessageBox(NULL,"Couldn't Perform Offset Calibration","Port Error",MB_ICONEXCLAMATION + MB_OK);
	 }

  }

  if (Ch == 'F')
  {
	 if (!FullScaleCal(M250Settings))
	 {
		MessageBox(NULL,"Couldn't Perform Full Scale Calibration","Port Error",MB_ICONEXCLAMATION + MB_OK);
	 }

  }

  if (Ch == 'U')
  {
	 M250Settings->digOutWord = -1;

	 while ((M250Settings->digOutWord < 0) || (M250Settings->digOutWord > 255))
	 {
		gotoxy(1,24);
		printf ("Enter Output Code ");
		fgets(tmpbuf,80,stdin);
		sscanf(tmpbuf,"%d",&M250Settings->digOutWord);
	 }

	 if (!SetDigOut(M250Settings->digOutWord))
	 {
		MessageBox(NULL,"Couldn't Set Digital Output","Port Error",MB_ICONEXCLAMATION + MB_OK);
	 }

  }

  if (Ch == ' ')
  {
	 (*max) = -12000.0;
	 (*min) = 12000.0;   //reset max-min
  }

  if ((Ch == 'L') || (Ch == 'A') || (Ch == 'C') || (Ch == 'E') || (Ch == 'U'))
  {
	 clrscr();
  }

  PrintCommandMenu(M250Settings, Ch, (*localAvg));
}


void LoadCalibrations(T_M250Settings far *M250Settings, char *currentCalStr)
{
int factErr = 0,calErr = 0, n;
char factCalStr[13]="FACT0000.CAL\0";
char newCalStr[13], Ch=0;

  while(Ch != 27)
  {
	 clrscr();
	 gotoxy(20,24);
	 printf("Current calibration ");

	 if (currentCalStr[3] == ' ')
	 {
		calErr = 1;
	 }

	 if (calErr)
	 {
		printf ("Uncalibrated");
	 }
	 else
	 {

		for(n=3;n<8;n++)
		{
		  printf ("%c",currentCalStr[n]);
		}

		if (factErr)
		{
		  gotoxy(19,25);
		  printf("No factory calibration found");
		}

	 }

	 gotoxy(21,6);
	 printf("Long-term Calibration Menu");
	 gotoxy(20,10);
	 printf("1) Load calibration file");
	 gotoxy(18,14);
	 printf("Esc) Return to Main Menu");

	 Ch = getch();

	 if (Ch == '1')
	 {
		ChangeId(newCalStr, currentCalStr);

		for (n = 4;n < 8;n++)
		{
		  currentCalStr[n]=newCalStr[n];
		}

		for (n = 4;n < 8;n++)
		{
		  factCalStr[n]=newCalStr[n];
		}

		printf ("LOADING CALIBRATION FILES\n");

		if (!GetCalibrations(M250Settings, &newCalStr[0]))
		{
			goto FileError;       //different error handler
		}

		calErr = 0;
		factErr = 0;
		strcpy (currentCalStr, newCalStr);
		goto LOADCALEXIT;

FileError:                //error handler for file errors

		if ((M250Settings->errorType&0xFF) != 0)
		{
		  clrscr();
		  calErr = 1;

		  if((M250Settings->errorType&0xFF) == 1)
		  {
			 printf("CALIBRATION FILE %s NOT FOUND \n",newCalStr);
		  }
		  else
		  {
			  printf ("%s WRONG SIZE, NOT LOADED \n",newCalStr);
		  }

		  printf("Accuracy can not be guaranteed\n");
		  printf("\n\nSelect long-term calibration to load calibration files\n");
		  printf("press ESC to continue\n");
		  Ch = 0;

		  while (Ch != 27)
		  {
			 Ch = getch();
		  }

		  Ch = 'X';
		}

		if(M250Settings->errorType > 0xFF)
		{
		  clrscr();
		  factErr = 1;

		  if((M250Settings->errorType>>8) == 1)
		  {
			 printf("FACTORY CAL FILE %s NOT FOUND \n",factCalStr);
		  }
		  else
		  {
			 printf ("%s WRONG SIZE, NOT LOADED \n",factCalStr);
		  }

		  printf("Accuracy can not be guaranteed\n");
		  printf("\n\nSelect long-term calibration to load calibration files\n");
		  printf("press ESC to continue\n");
		  Ch = 0;

		  while (Ch != 27)
		  {
			 Ch = getch();
		  }

		  Ch = 'X';
		}

LOADCALEXIT:
	 }

  }

}

void ChangeId(char *newCalStr, char *currentCalStr)
{
char tmpbuf[80];
int n;

  clrscr();
  strcpy (newCalStr, currentCalStr);
  printf("Current calibration file name:  ");

  if (currentCalStr[3] == ' ')
  {
	 printf("NONE");
  }
  else
  {
	 printf("%s",currentCalStr);
  }

  gotoxy(1,3);
  printf("Enter one letter plus 4 digit serial number \n");
  printf ("(or return to leave unchanged) ");
  gets(tmpbuf);

  if(strlen(tmpbuf))
  {

	 if (islower(tmpbuf[0]))
	 {
		tmpbuf[0] = toupper(tmpbuf[0]);
	 }

	 if (tmpbuf[0] >= 'A' && tmpbuf[0] <= 'Z')
	 {
		newCalStr[3] = tmpbuf[0];

		for(n = 1;n < 5;n++)
		{

		  if (tmpbuf[n] >= '0' && tmpbuf[n] <= '9')
		  {
			 newCalStr[3+n] = tmpbuf[n];
		  }

		}

	 }
  }
  else
  {
	 strcpy(newCalStr, "");
  }
}


void SetWindowToDesktop(void)  //make window the same size as the desktop
{
RECT rect;

	 clrscr();
	 GetWindowRect(GetDesktopWindow(), &rect);
	 SetWindowPos(GetActiveWindow(), (HWND) NULL, 0, 0,
	 rect.right, rect.bottom,
	 SWP_NOZORDER | SWP_NOACTIVATE);
}

short   WriteConfig(T_M250Settings far *M250Settings,
						  short *localAvg,
						  char *newCalStr)

{
FILE *out;

  if ((out = fopen("250dll.cfg", "wt")) != NULL)
  {
	 fprintf(out,"%d\n",M250Settings->portAddress);
	 fprintf(out,"%d\n",M250Settings->rwDelay);
	 fprintf(out,"%lf\n",M250Settings->sIntvl);
	 fprintf(out,"%ld\n",M250Settings->stampBuffer);
	 fprintf(out,"%d\n",M250Settings->interChanDelay);
	 fprintf(out,"%d\n",M250Settings->numChannels);
	 fprintf(out,"%d\n",M250Settings->average);
	 fprintf(out,"%d\n",M250Settings->tScan);
	 fprintf(out,"%d\n",M250Settings->chan[0]);
	 fprintf(out,"%d\n",M250Settings->chan[1]);
	 fprintf(out,"%d\n",M250Settings->chan[2]);
	 fprintf(out,"%d\n",M250Settings->chan[3]);
	 fprintf(out,"%d\n",M250Settings->chan[4]);
	 fprintf(out,"%d\n",M250Settings->chan[5]);
	 fprintf(out,"%d\n",M250Settings->triggerType);
	 fprintf(out,"%lf\n",M250Settings->gainFactor);
	 fprintf(out,"%lf\n",M250Settings->plusGain);
	 fprintf(out,"%lf\n",M250Settings->minusGain);
	 fprintf(out,"%d\n",M250Settings->trackHold);
	 fprintf(out,"%d\n",M250Settings->channel);
	 fprintf(out,"%d\n",M250Settings->subChan);
	 fprintf(out,"%d\n",localAvg);
	 fprintf(out,"%s\n",newCalStr);
	 fclose(out);
  }

  return 1;

}

short   ReadConfig(T_M250Settings far *M250Settings,
						 short *localAvg,
						 char *newCalStr)

{
char tmpbuf[80];
FILE *in;

  if ((in = fopen("250dll.cfg", "rt")) != NULL)
  {
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->portAddress);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->rwDelay);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%lf",&M250Settings->sIntvl);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%ld",&M250Settings->stampBuffer);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->interChanDelay);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->numChannels);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->average);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->tScan);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->chan[0]);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->chan[1]);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->chan[2]);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->chan[3]);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->chan[4]);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->chan[5]);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->triggerType);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%lf",&M250Settings->gainFactor);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%lf",&M250Settings->plusGain);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%lf",&M250Settings->minusGain);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->trackHold);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->channel);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&M250Settings->subChan);
	 fgets(tmpbuf,80,in);
	 sscanf(tmpbuf,"%d",&localAvg);
	 fgets(newCalStr,80,in);
	 fclose(in);
	 return 1;
  }
  else
  {
	 //stampBuffer = 1 because we are only getting one scan at a time
	 M250Settings->stampBuffer = 1;
	 M250Settings->numChannels = 5;
	 M250Settings->portAddress = 0x378;
	 M250Settings->rwDelay = 0;
	 M250Settings->average = 1;
	 M250Settings->trackHold = HOLD;
	 M250Settings->channel= 7;
	 M250Settings->subChan= 0;
	 M250Settings->chan[0] = 0;
	 M250Settings->chan[1] = 0;
	 M250Settings->chan[2] = 0;
	 M250Settings->chan[3] = 0;
	 M250Settings->chan[4] = 0x80;
	 M250Settings->chan[5] = 0x80;
	 M250Settings->tScan = 1;
	 M250Settings->triggerType = 0;
	 M250Settings->interChanDelay = 20;
	 M250Settings->sIntvl = .0088183;
	 return 0;
  }

}

void LogData(T_M250Settings far *M250Settings,
				 int numberOfScans,
				 int *i,
				 short *cc,
				 long far *ScanBuffer)

{
int                  n, j;
int 						startCC, endCC, ccode;
long 						timestamp,counts;
double					voltage;

	n = 0;

	while (n < numberOfScans)
	{
	  if ((*i) == (M250Settings->stampBuffer*(M250Settings->numChannels + 1)))
	  {
		 (*i) = 0;
	  }

	  timestamp = ScanBuffer[(*i)];

	  fprintf(out, "time: %11.4lf", (timestamp-1)*M250Settings->sIntvl);

	  if (M250Settings->tScan == 1)
	  {
		 (*i)++;

		 if ((*i) == (M250Settings->stampBuffer*(M250Settings->numChannels + 1)))
		 {
			(*i) = 0;
		 }

		 counts = ScanBuffer[(*i)];
		 voltage = counts * M250Settings->gainFactor *.001192093;

		 if (M250Settings->trackHold)
		 {

			if (voltage < 0)
			{
			  voltage *= M250Settings->minusGain;
			}
			else
			{
			  voltage *= M250Settings->plusGain;
			}

		 }

		 fprintf(out, "Channel 6 %9.3lf ",voltage);
	  }  //end if time stamp scan with cal

	  for (j = 0;j<= 5;j++)           //get # of datapoints for each channel
	  {
		 startCC = M250Settings->chan[j]>>4;
		 endCC = cc[j] + startCC - 1;
		 for (ccode = startCC;ccode<= endCC; ccode++)
		 {
			 (*i)++;

			 if ((*i) == (M250Settings->stampBuffer*(M250Settings->numChannels + 1)))
			 {
				(*i) = 0;
			 }
			 counts = ScanBuffer[(*i)];
			 voltage = counts * M250Settings->gainFactor *.001192093;

			 if (M250Settings->trackHold)
			 {

				if (voltage < 0)
				{
				  voltage *= M250Settings->minusGain;
				}
				else
				{
					voltage *= M250Settings->plusGain;
				}

			 }

			 fprintf(out, " %9.3lf ",voltage);
		 } //end control code for loop

	  } //end channel for loop

	  n++;
	  (*i)++;
	  fprintf(out,"\n");

	}  //end while m\n < numberOfScans
}

